﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class GameManager : MonoBehaviour {

	public enum GameState{MainMenu, GameRunning, GamePaused, EndScreen, Setup};
	public GameState currentState;

	private static GameManager instance;

	public InstantiateList gamePrefabs;
	private InstantiateList gp;

	public Transform gameUI;
	private Transform _gameUI;
	public Transform pauseUI;
	private Transform _pauseUI;
	public Transform titleUI;
	private Transform _titleUI;
	public Transform endUI;
	private Transform _endUI;
	//public string gameUISearchString = "GameUI";

	public MainCameraScript mainCamera;

	public float startHealth = 3;
	private float health;
	public float maxHealth = 5;
	public float dieHealth = 0;
	public Mask healthMask;
	public string healthMaskSearchString = "Panel/HealthLabel/HealthMask";
	private float healthMaskStartWidth;

	public string scorePrefix = "Height:\n";
	public string scoreSuffix = "m";
	public string scoreFormat = "n1";
	public Text scoreText;
	public string scoreTextSearchString = "Panel/ScoreText";
	private float maxScore;

	public string bestScorePrefix = "Best:\n";
	public string bestScoreSuffix = "m";
	public string bestScoreFormat = "n0";
	public Text bestScoreText;
	public string bestScoreTextSearchString = "Panel/BestScore";
	private float bestScore = 0;

	public string moveSpeedPrefix = "Speed:\n";
	public string moveSpeedSuffix = "%";
	public string moveSpeedFormat = "n0";
	public Text moveSpeedText;
	public string moveSpeedTextSearchString = "Panel/MoveSpeed";

	public string jumpSpeedPrefix = "Jump:\n";
	public string jumpSpeedSuffix = "%";
	public string jumpSpeedFormat = "n0";
	public Text jumpSpeedText;
	public string jumpSpeedTextSearchString = "Panel/JumpSpeed";

	public string endDeadMessage = "You died... : (";
	private Text endDeadText;
	public string endDeadMessageSearchString = "YouDied";

	public string endScorePrefix = "You reached ";
	public string endScoreSuffix = "m";
	public string endScoreFormat = "n1";
	private Text endScoreText;
	public string endScoreTextSearchString = "YouDied/FinalScore";

	public string endBestPrefix = "Your best is ";
	public string endBestSuffix = "m";
	public string endBestFormat = "n0";
	private Text endBestText;
	public string endBestTextSearchString = "YouDied/FinalScore/BestScore";

	public string endBottomMessage = "Thanks for playing! <color=#0fa0d0ff>slasheethecow.com</color>";
	private Text endBottomText;
	public string endBottomMessageSearchString = "ThanksForPlaying";


	private float targetTime, normalTime = 1, storedTimeScale;
	private bool slowingDown = false, slowed = false, slowingUp = false;
	private float slowStartTime, unslowStartTime, slowedTime, transitionTime;

	public Transform gameMusic, menuMusic;
	private SoundScript _gameMusic, _menuMusic;

	// Use this for initialization
	void Start () {
		if(instance == null){
			instance = this;
		}
		currentState = GameState.Setup; //it's a special state, please don't do anything in it
		musicSetup();
		setupMainMenu();
		//gameSetup();
	}

	private void musicSetup(){
		if(gameMusic != null){
			Transform t = (Transform) Instantiate(gameMusic);
			_gameMusic = t.GetComponent<SoundScript>();
			//_gameMusic.pause();
		}

		if(menuMusic != null){
			Transform t = (Transform) Instantiate(menuMusic);
			_menuMusic = t.GetComponent<SoundScript>();
			//_menuMusic.pause();
		}
	}

	private void setupMainMenu(){
		currentState = GameState.MainMenu;
		Time.timeScale = normalTime;
		if(_menuMusic != null) _menuMusic.play();
		//if(_menuMusic != null) print ("menu music not null");
		_titleUI = (Transform) Instantiate(titleUI);
	}

	private void endCurrentState(){
		switch(currentState){
		case GameState.MainMenu:
			if(_menuMusic != null) _menuMusic.pause();
			if(_titleUI != null) Destroy(_titleUI.gameObject);
			break;
		case GameState.GameRunning:
			gp.destroyAll();
			if(gp != null) Destroy(gp.gameObject);
			if(_gameMusic != null) _gameMusic.pause();
			if(_gameUI != null) Destroy(_gameUI.gameObject);
			break;
		case GameState.GamePaused:
			gp.destroyAll();
			if(gp != null) Destroy(gp.gameObject);
			if(_menuMusic != null) _menuMusic.pause();
			if(_gameUI != null) Destroy(_gameUI.gameObject);
			if(_pauseUI != null) Destroy(_pauseUI.gameObject);
			break;
		case GameState.EndScreen:
			if(_menuMusic != null) _menuMusic.pause();
			if(_endUI != null) Destroy(_endUI.gameObject);
			break;
		case GameState.Setup:
			break;
		}
	}

	public static void startGame(){
		//if(instance._titleUI != null) Destroy(instance._titleUI.gameObject);
		//if(instance._endUI != null) Destroy(instance._endUI.gameObject);
		instance.endCurrentState();
		instance.gameSetup();
	}

	private void gameSetup(){
		if(instance._gameMusic != null) instance._gameMusic.play();
		findGameUI();
		if(mainCamera != null) mainCamera.resetCamera();
		if(gamePrefabs != null){
			gp = (InstantiateList) Instantiate(gamePrefabs);
			gp.instantiateAll();
			//gp.GetComponent<InstantiateList>().runInstantiations();
		}
		//if(gameUI != null) Instantiate(gameUI);
		health = startHealth;
		healthMaskStartWidth = healthMask.rectTransform.rect.width;
		changeHealth(0);
		recordScore(0);
		maxScore = 0;
		Time.timeScale = normalTime;
		targetTime = normalTime;
		slowingUp = false; slowed = false; slowingDown = false;
		setBest();
		currentState = GameState.GameRunning;
	}

	private void findGameUI(){
		/*if(gameUI == null) gameUI = GameObject.Find(gameUISearchString).transform;
		if(healthMask == null) healthMask = GameObject.Find(healthMashSearchString).GetComponent<Mask>();
		if(scoreText == null) scoreText = GameObject.Find(scoreTextSearchString).GetComponent<Text>();
		if(bestScoreText == null) bestScoreText = GameObject.Find(bestScoreTextSearchString).GetComponent<Text>();
		if(moveSpeedText == null) moveSpeedText = GameObject.Find(moveSpeedTextSearchString).GetComponent<Text>();
		if(jumpSpeedText == null) jumpSpeedText = GameObject.Find(jumpSpeedTextSearchString).GetComponent<Text>();*/
		_gameUI = (Transform) Instantiate(gameUI);
		healthMask = _gameUI.FindChild(healthMaskSearchString).GetComponent<Mask>();
		scoreText = _gameUI.FindChild(scoreTextSearchString).GetComponent<Text>();
		bestScoreText = _gameUI.FindChild(bestScoreTextSearchString).GetComponent<Text>();
		moveSpeedText = _gameUI.FindChild(moveSpeedTextSearchString).GetComponent<Text>();
		jumpSpeedText = _gameUI.FindChild(jumpSpeedTextSearchString).GetComponent<Text>();
	}

	private void findEndUI(){
		_endUI = (Transform) Instantiate(endUI);
		endDeadText = _endUI.FindChild(endDeadMessageSearchString).GetComponent<Text>();
		endScoreText = _endUI.FindChild(endScoreTextSearchString).GetComponent<Text>();
		endBestText = _endUI.FindChild(endBestTextSearchString).GetComponent<Text>();
		endBottomText = _endUI.FindChild(endBottomMessageSearchString).GetComponent<Text>();
	}

	private void setupEndUI(){
		currentState = GameState.EndScreen;
		endDeadText.text = endDeadMessage;
		endScoreText.text = endScorePrefix + maxScore.ToString(endScoreFormat) + endScoreSuffix;
		endBestText.text = endBestPrefix + bestScore.ToString(endBestFormat) + endBestSuffix;
		endBottomText.text = endBottomMessage;
		//currentState = GameState.EndScreen;
	}
	
	// Update is called once per frame
	void Update () {
		switch(currentState){
		case GameState.MainMenu: {
			break;
			}
		case GameState.GameRunning: {
			if(slowingDown){
				float t = (Time.time - slowStartTime) / transitionTime;
				if(t >= 1){
					Time.timeScale = targetTime;
					unslowStartTime = Time.time + (slowedTime * targetTime);
					slowingDown = false;
					slowed = true;
				} else {
					Time.timeScale = Mathf.SmoothStep(normalTime, targetTime, t);
				}
			} else if(slowed){
				if(Time.time > unslowStartTime){
					slowed = false;
					slowingUp = true;
					unslowStartTime = Time.time;
				}
			} else if(slowingUp){
				float t = (Time.time - unslowStartTime) / transitionTime;
				if(t >= 1){
					Time.timeScale = normalTime;
					slowingUp = false;
				} else {
					Time.timeScale = Mathf.SmoothStep(transitionTime, normalTime, t);
				}
			}

			if(Input.GetButtonDown("Cancel")) pauseGame();

			break;
		}
		case GameState.GamePaused: {
			if(Input.GetButtonDown("Cancel")) unpauseGame();
			break;
		}
		case GameState.EndScreen: {
			break;
		}
	}
	}

	public static void pauseGame(){
		instance.storedTimeScale = Time.timeScale;
		Time.timeScale = 0;
		instance.currentState = GameState.GamePaused;
		instance._gameUI.gameObject.SetActive(false);
		if(instance._gameMusic != null) instance._gameMusic.pause();
		if(instance._menuMusic != null) instance._menuMusic.play();
		instance._pauseUI = (Transform) Instantiate(instance.pauseUI);
	}

	public static void unpauseGame(){
		Time.timeScale = instance.storedTimeScale;
		instance.currentState = GameState.GameRunning;
		instance._gameUI.gameObject.SetActive(true);
		if(instance._gameMusic != null) instance._gameMusic.play();
		if(instance._menuMusic != null) instance._menuMusic.pause();
		Destroy(instance._pauseUI.gameObject);
	}

	public static void endGame(){
		instance.endCurrentState();
		instance.currentState = GameState.EndScreen;
		if(instance._menuMusic != null) instance. _menuMusic.play();
		instance.findEndUI();
		instance.setupEndUI();
	}

	public static void mainMenu(){
		instance.endCurrentState();
		instance.currentState = GameState.MainMenu;
		instance.setupMainMenu();
	}

	public static bool changeHealth(float change){
		//returns false if already at max or health loss causes death
		instance.health += change;
		instance.healthMask.rectTransform.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, instance.health / instance.maxHealth * instance.healthMaskStartWidth);
		if(instance.health <= instance.dieHealth){
			endGame();
			return false;
		} else if (instance.health > instance.maxHealth){
			instance.health = instance.maxHealth;
			return false;
		} else if (instance.health == instance.maxHealth){
			return false;
		}
		return true;
	}
	
	public static void recordScore(float score){
		if (score > instance.maxScore){
			instance.maxScore = score;
			if(instance.maxScore > instance.bestScore){
				instance.bestScore = instance.maxScore;
				setBest();
			}
			instance.scoreText.text = instance.scorePrefix + instance.maxScore.ToString(instance.scoreFormat) + instance.scoreSuffix;
		}
	}
	
	private static void setBest(){
		instance.bestScoreText.text = instance.bestScorePrefix + instance.bestScore.ToString(instance.bestScoreFormat) + instance.bestScoreSuffix;
	}

	public static void setMoveSpeed(float current, float min){
		instance.moveSpeedText.text = instance.moveSpeedPrefix + (current / min * 100).ToString(instance.moveSpeedFormat) + instance.moveSpeedSuffix;
	}

	public static void setJumpSpeed(float current, float min){
		instance.jumpSpeedText.text = instance.jumpSpeedPrefix + (current / min * 100).ToString(instance.jumpSpeedFormat) + instance.jumpSpeedSuffix;
	}

	public static void slowTime(float targetTimeScale, float slowedTime, float transitionTime){
		instance.targetTime = targetTimeScale; instance.slowedTime = slowedTime; instance.transitionTime = transitionTime;
		instance.slowStartTime = Time.time; instance.slowingDown = true;
	}

	public static bool currentlySlowed(){
		return instance.slowingDown || instance.slowed || instance.slowingUp;
	}

	public static bool gameRunning(){
		if(instance.currentState == GameState.GameRunning){return true;}
		else {return false;}
	}

	public static void quitGame(){
		Application.Quit();
	}
}
