using UnityEngine;
using System.Collections;

public class PlatformsRiser : MonoBehaviour {

	public int platsX;
	public int platsY;
	public Transform platformPrefab;
	public Vector2 platformScale = new Vector2(1,1); //will scale x and z of platform to this
	public Vector2 platformSize = new Vector2(1,1); //x and y gaps between platforms
	private Transform[,] platforms;

	public float riseTriggerTickTime = 1;
	private float nextRiseTick;
	private float lastRiseTick;
	public float risePlatformsPerTick;
	public float riseRatePerSecond;
	public float riseRateIncreaseRate;
	private float riseLevel;
	public float startLevel;
	public float riseToWithin = 1; //platforms will rise to riseLevel +/- this number

	private static PlatformsRiser instance;

	// Use this for initialization
	void Start () {
		instance = this;
		platforms = new Transform[platsX,platsY];
		for(int x = 0; x < platsX; x++){
			for(int y = 0; y < platsY; y++){
				Transform newPlatform = Instantiate(platformPrefab, new Vector3(transform.position.x + x * platformSize.x, transform.position.y, transform.position.z + y * platformSize.y), Quaternion.identity) as Transform;
				newPlatform.SetParent(this.transform);
				newPlatform.transform.localScale = new Vector3(platformScale.x, newPlatform.localScale.y, platformScale.y);
				newPlatform.name = "platform(" + x + "," + y + ")";
				platforms[x,y] = newPlatform;
			}
		}

		riseLevel = startLevel;
		lastRiseTick = Time.time;
		nextRiseTick = Time.time + riseTriggerTickTime;
	}
	
	// Update is called once per frame
	void Update () {
		if(GameManager.gameRunning()){
			if(Time.time >= nextRiseTick){
				//calculate the new level we're at
				riseLevel += riseRatePerSecond * (Time.time - lastRiseTick);
				//riseLevel *= 1 + (riseRatePerSecond * (Time.time - lastRiseTick));
				//print (riseRatePerSecond * (Time.time - lastRiseTick));
				//print (Time.time - lastRiseTick);
				//increase the rate at which it rises
				riseRatePerSecond *= 1 + (riseRateIncreaseRate * (Time.time - lastRiseTick));
				//riseLevel += riseRatePerSecond * (Time.time - lastRiseTick);

				int r = 0; //it's short for "platforms raised", for what that's worth
				while(r < risePlatformsPerTick){
					int x = Random.Range((int) 0, (int) platsX);
					int y = Random.Range((int) 0, (int) platsY);
					//if(!platforms[x,y].GetComponent<PlatformScript>().rising()){
					if(platforms[x,y].GetComponent<PlatformScript>().setRise(Random.Range(riseLevel - riseToWithin, riseLevel + riseToWithin))){
						r++;
						//print("Raised at " + Time.time + " R:" + r);
					}
					//}
				}

				//schedule the next tick
				nextRiseTick = lastRiseTick + riseTriggerTickTime * 2;
				//nextRiseTick = Time.time + riseTriggerTickTime;
				//print(Time.time + riseTriggerTickTime);
				//print(lastRiseTick + riseTriggerTickTime);
				lastRiseTick = Time.time;
			}
		}
	}

	public static int platformsX(){return instance.platsX;}
	public static int platformsY(){return instance.platsY;}
	public static Transform getPlatform(int x, int y){return instance.platforms[x,y];}
	public static bool anyPlatformsAboveLava(float gapNeeded = 0){
		foreach(Transform platform in instance.platforms){
			if(platform.GetComponent<PlatformScript>().headAboveWater(gapNeeded)) return true;
			//print ("checking platform " + Time.time);
		}
		return false;
	}
	public static Vector2 getHalfSize(){return instance.platformSize / 2;}
}
