﻿using UnityEngine;
using System.Collections;

[RequireComponent(typeof(CharacterController))]
public class PlayerScript : MonoBehaviour {

	public string platformMagicWord = "platform";
	public string groundMagicWord = "ground";
	public string lavaMagicWord = "lava";
	public string pickupMagicWord = "pickup";

	//public string visibleTransform;

	public float startMoveSpeed = 4;
	private float _moveSpeed;
	private float moveSpeed{
		get{return _moveSpeed;}
		set{_moveSpeed = value; GameManager.setMoveSpeed(moveSpeed, startMoveSpeed);}
	}
	public float startJumpSpeed = 4;
	private float _jumpSpeed;
	private float jumpSpeed{
		get{return _jumpSpeed;}
		set{_jumpSpeed = value; GameManager.setJumpSpeed(jumpSpeed, startJumpSpeed);}
	}
	public float gravity = 10;
	private CharacterController controller;
	private Vector3 moveDirection;

	private Transform currentPlatform;
	private Transform lastPlatform;
	private float medHeight;
	private float lastPlatformHeight;

	public float lavaJumpSpeed = 20;
	public float lavaHealth = -1;
	private bool justHitLava = false;

	public float healthPickupHealth = 1;
	public Transform healthPickupMaxText;
	public Transform healthPickupSound;

	public float jumpPickupIncrease = .5f;
	public float maxJumpSpeed = 8;
	public Transform jumpPickupMaxText;
	public Transform jumpPickupSound;

	public float speedPickupIncrease = .5f;
	public float maxMoveSpeed = 8;
	public Transform speedPickupMaxText;
	public Transform speedPickupSound;

	public float slowPickupTimeScale = .5f;
	public float slowPickupSlowdownTime = 4;
	public float slowPickupTransitionTime = .5f;
	public Transform slowPickupMaxText;
	public Transform slowPickupSound;

	public Transform HurtSound;
	public Transform JumpSound;

	// Use this for initialization
	void Start () {
		controller = GetComponent<CharacterController>();
		moveDirection = new Vector3(0,0,0);
		medHeight = collider.bounds.extents.y;
		moveSpeed = startMoveSpeed;
		jumpSpeed = startJumpSpeed;
	}
	
	// Update is called once per frame
	void Update () {
		if(GameManager.gameRunning()){
			moveDirection = new Vector3(Input.GetAxis("Horizontal"), controller.isGrounded ? 0 : moveDirection.y, Input.GetAxis("Vertical"));
			//moveDirection = transform.TransformDirection(moveDirection);
			moveDirection = new Vector3(moveDirection.x * moveSpeed, moveDirection.y, moveDirection.z * moveSpeed);
			if(Input.GetButtonDown("Jump") && controller.isGrounded){
			//if(Input.GetButtonDown("Jump") && Physics.Raycast(transform.position, Vector3.down, collider.bounds.extents.y + .1f)){
				moveDirection.y = jumpSpeed;
				lastPlatform = currentPlatform;
				currentPlatform = null;
				if(JumpSound != null) Instantiate(JumpSound, transform.position, transform.rotation);
			}
			if(justHitLava){
				moveDirection.y = lavaJumpSpeed;
				justHitLava = false;
			}
			moveDirection.y -= gravity * Time.deltaTime;
			controller.Move(moveDirection * Time.deltaTime);

			//here's my favourite part: rotate so we're the right way around
			if(Mathf.Abs(Input.GetAxis("Horizontal")) == Mathf.Abs(Input.GetAxis("Vertical"))){ //they're either maxed out, nothing, or exactly the same, whatever way, don't make me choose
			} else if (Mathf.Abs(Input.GetAxis("Horizontal")) > Mathf.Abs(Input.GetAxis("Vertical"))){ //horizontal input wins!
				transform.eulerAngles = new Vector3(0, Input.GetAxis("Horizontal") > 0 ? 90 : 270, 0);
			} else { //vertical must be the only thing left
				transform.eulerAngles = new Vector3(0, Input.GetAxis("Vertical") > 0 ? 0 : 180, 0);
			}

			//tell the manager how high we are
			if(controller.isGrounded) GameManager.recordScore(collider.bounds.min.y);

			/*if(Input.GetButtonDown("Jump")){
				print ("jump!");
				rigidbody.AddForce(jumpAcceleration * Vector3.up, ForceMode.VelocityChange);
			}*/
		}
	}

	void LateUpdate(){
		if(currentPlatform != null){
			if(!controller.isGrounded){
				lastPlatform = currentPlatform;
				currentPlatform = null;
			} else if(currentPlatform.collider.bounds.max.y > lastPlatformHeight){
				transform.Translate((currentPlatform.collider.bounds.max.y - lastPlatformHeight) * 1 * Vector3.up);
				//transform.position = new Vector3(transform.position.x, currentPlatform.collider.bounds.max.y + medHeight, transform.position.z);
				lastPlatformHeight = currentPlatform.collider.bounds.max.y;
			}
		}

		//brute force solution to make sure you'll go up with platforms even when they go up real fast
		if(lastPlatform != null){
			if(lastPlatform.collider.bounds.Contains(new Vector3(transform.position.x, collider.bounds.min.y, transform.position.z)) /*&& lastPlatform.collider.bounds.max.y > collider.bounds.min.y*/){

				transform.position = new Vector3(transform.position.x, lastPlatform.collider.bounds.max.y + medHeight, transform.position.z);
			}
		}
	}

	//void OnCollisionEnter(Collision hit){
	void OnControllerColliderHit(ControllerColliderHit hit){
		if(hit.collider.gameObject.name.ToUpper().Contains(platformMagicWord.ToUpper())){ //we're colliding with a platform
			currentPlatform = hit.collider.transform;
			lastPlatformHeight = hit.collider.bounds.max.y;
		}
		if(hit.collider.gameObject.name.ToUpper().Contains(groundMagicWord.ToUpper())){
			lastPlatform = currentPlatform;
			currentPlatform = null;
		}
		/*if(hit.collider.gameObject.name.ToUpper().Contains(lavaMagicWord.ToUpper())){
			print("lava");
		}*/
		if (hit.collider.gameObject.name.ToUpper().Contains(pickupMagicWord.ToUpper())){
			getPickup(hit.collider.GetComponent<PickupScript>());
		}
	}

	void OnTriggerEnter(Collider other){
		if(other.gameObject.name.ToUpper().Contains(lavaMagicWord.ToUpper())){
			justHitLava = true;
			GameManager.changeHealth(lavaHealth);
			Instantiate(HurtSound, transform.position, transform.rotation);
			moveSpeed = startMoveSpeed;
			jumpSpeed = startJumpSpeed;
		}
	}

	void getPickup(PickupScript pickup){
		switch(pickup.type){
			case PickupScript.PickupType.Health: {
				if(!GameManager.changeHealth(healthPickupHealth)){
					Instantiate(healthPickupMaxText, pickup.transform.position, Quaternion.Euler(90,0,0));
				}
				if(healthPickupSound != null) Instantiate(healthPickupSound, transform.position, transform.rotation);
				break;
			}
			case PickupScript.PickupType.Jump: {
				if(jumpSpeed == maxJumpSpeed){Instantiate(jumpPickupMaxText, pickup.transform.position, Quaternion.Euler(90,0,0)); jumpSpeed = maxJumpSpeed; break;}
					jumpSpeed += jumpPickupIncrease;
				if(jumpPickupSound != null) Instantiate(jumpPickupSound, transform.position, transform.rotation);
				if(jumpSpeed >= maxJumpSpeed){Instantiate(jumpPickupMaxText, pickup.transform.position, Quaternion.Euler(90,0,0)); jumpSpeed = maxJumpSpeed;}
				break;
			}
			case PickupScript.PickupType.Speed: {
				if(moveSpeed == maxMoveSpeed){Instantiate(speedPickupMaxText, pickup.transform.position, Quaternion.Euler(90,0,0)); moveSpeed = maxMoveSpeed; break;}
				moveSpeed += speedPickupIncrease;
				if(speedPickupSound != null) Instantiate(speedPickupSound, transform.position, transform.rotation);
				if(moveSpeed >= maxMoveSpeed){Instantiate(speedPickupMaxText, pickup.transform.position, Quaternion.Euler(90,0,0)); moveSpeed = maxMoveSpeed;}
				break;
			}
			case PickupScript.PickupType.Slow: {
				if(!GameManager.currentlySlowed()){
					GameManager.slowTime(slowPickupTimeScale, slowPickupSlowdownTime, slowPickupTransitionTime);
					if(slowPickupSound != null) Instantiate(slowPickupSound, transform.position, transform.rotation);
				} else {
					Instantiate(slowPickupMaxText, pickup.transform.position, Quaternion.Euler(90,0,0)); moveSpeed = maxMoveSpeed;
				}
				break;
			}
		}
		Destroy(pickup.gameObject);
	}
}
